@@ -13,8 +13,7 @@ export = oclexport.OclExportFactory.load_latest_export(
 
 # Load export from JSON file
 export_filename = 'my-export.json'
-export = oclexport.OclExportFactory.load_export(
-    repo_version_url=repo_version_url, oclapitoken=my_ocl_api_token)
+export = oclexport.OclExportFactory.load_from_export_json_file(export_filename)
 
 # Get concepts from export that include mappings
 concept_a = export.get_concept_by_index(
@@ -35,9 +34,10 @@ pprint.pprint(concept_list)
 """
 import json
 import zipfile
-from StringIO import StringIO
 import requests
-import oclresourcelist
+import six
+
+from ocldev import oclconstants
 
 
 class OclError(Exception):
@@ -48,6 +48,7 @@ class OclError(Exception):
 class OclUnknownResourceError(OclError):
     """ Error thrown when an unknown resource type is encountered """
     def __init__(self, expr, msg):
+        OclError.__init__(self)
         self.expr = expr
         self.msg = msg
 
@@ -55,12 +56,13 @@ class OclUnknownResourceError(OclError):
 class OclExportNotAvailableError(OclError):
     """ Error thrown when requesting an OCL export that is not available """
     def __init__(self, expr, msg):
+        OclError.__init__(self)
         self.expr = expr
         self.msg = msg
 
 
 class OclExportFactory(object):
-    """ Factory class to create OclExport factory objects """   
+    """ Factory class to create OclExport factory objects """
 
     @staticmethod
     def load_export(repo_version_url='', oclapitoken=''):
@@ -86,7 +88,12 @@ class OclExportFactory(object):
 
         # Decompress "export.json" from the zipfile in memory and return as a python dictionary
         repo_export = None
-        export_string_handle = StringIO(r.content)
+        content = r.content
+        if isinstance(content, bytes):
+            export_string_handle = six.BytesIO(content)
+        else:
+            export_string_handle = six.StringIO(content)
+
         zipref = zipfile.ZipFile(export_string_handle, "r")
         if 'export.json' in zipref.namelist():
             repo_export = json.loads(zipref.read('export.json'))
@@ -94,7 +101,7 @@ class OclExportFactory(object):
         else:
             zipref.close()
             errmsg = 'ERROR: Invalid repository export for "%s": ' % repo_version_url
-            errmsg += 'export.json not found in the export response.\n%s' % r.content
+            errmsg += 'export.json not found in the export response.\n%s' % content
             raise Exception(errmsg)
         return OclExport(repo_export)
 
@@ -113,7 +120,9 @@ class OclExportFactory(object):
     @staticmethod
     def get_latest_version_id(repo_url, oclapitoken=''):
         """
-        Get the ID of the most recent released version of the specified repository
+        Get the ID of the most recent released version of the specified repository.
+        Repo URL should be of the format:
+        https://api.openconceptlab.org/orgs/MyOrg/sources/MySource/
         """
 
         # Prepare the request headers
@@ -141,6 +150,8 @@ class OclExportFactory(object):
 class OclExport(object):
     """ Object representing an OCL export of an source or collection version """
 
+    RESOURCE_PATTERN = r'^(\/(orgs|users)\/([a-zA-Z0-9\-\.\_\@]+)\/(sources|collections)\/([a-zA-Z0-9\-\.\_\@]+)\/(concepts|mappings)\/([a-zA-Z0-9\-\.\_\@]+)\/)(([a-zA-Z0-9\-\.\_\@]+)\/)?$'
+
     def __init__(self, export_json=None, ocl_export=None):
         """ Initialize this OclExport object """
         self._export_json = None
@@ -148,17 +159,118 @@ class OclExport(object):
         self._mappings = []
         self.set_export(export_json=export_json, ocl_export=ocl_export)
 
+    def __len__(self):
+        """ Number of resources in this list """
+        if self._export_json:
+            return len(self._export_json['concepts']) + len(self._export_json['mappings'])
+
     def get_full_export(self):
         """ Return full contents of export as a python dictionary """
         return self._export_json
 
-    def to_resource_list(self):
-        """ Return all concepts and mappings as an OclJsonResourceList """
+    def to_resource_list(self, do_clean_for_bulk_import=False, do_include_concepts=True,
+                         do_include_mappings=True, do_include_references=False,
+                         do_include_repo=False, do_include_repo_version=False):
+        """
+        Return all resources in an OclExport as an as an OclJsonResourceList. By default, only
+        concepts and mappings are included regardless of whether it is a Source or Collection
+        export. Use the arguments to include references, the repository and repository version,
+        or to omit concepts or mappings. If `do_clean_for_bulk_import` is true, only attributes
+        required to import an equivalent resource via bulk import are included in each resource.
+        """
         if not self._export_json:
             return None
+        from ocldev import oclresourcelist
         resource_list = oclresourcelist.OclJsonResourceList()
-        resource_list.append(self._concepts)
-        resource_list.append(self._mappings)
+
+        # Determine repository type
+        if (self._export_json['type'] in [
+                oclconstants.OclConstants.RESOURCE_TYPE_COLLECTION_VERSION,
+                oclconstants.OclConstants.RESOURCE_TYPE_COLLECTION]):
+            repo_type = oclconstants.OclConstants.RESOURCE_TYPE_COLLECTION
+        elif (self._export_json['type'] in [
+                oclconstants.OclConstants.RESOURCE_TYPE_SOURCE_VERSION,
+                oclconstants.OclConstants.RESOURCE_TYPE_SOURCE]):
+            repo_type = oclconstants.OclConstants.RESOURCE_TYPE_SOURCE
+        else:
+            errmsg = 'Invalid export type "%s". Expected Source Version or Collection Version.' % (
+                self._export_json['type'])
+            raise ValueError(errmsg)
+
+        # Add resource for the repository (source or collection)
+        if do_include_repo:
+            remove_attrs = ['active_concepts', 'active_mappings', 'concepts_url', 'created_by',
+                            'created_on', 'mappings_url', 'owner_url', 'updated_on', 'updated_by',
+                            'url', 'uuid', 'versions', 'versions_url']
+            if repo_type.lower() not in self._export_json:
+                raise ValueError('Expected "%s" field in export.' % repo_type.lower())
+            repo = dict(self._export_json[repo_type.lower()])
+            for attr_key in remove_attrs:
+                repo.pop(attr_key, '')
+            resource_list.append(repo)
+
+        # Add concepts
+        if do_include_concepts:
+            if do_clean_for_bulk_import:
+                allowed_attr_keys = [
+                    'concept_class', 'datatype', 'descriptions', 'external_id', 'extras', 'id',
+                    'names', 'owner', 'owner_type', 'retired', 'source', 'type']
+                for concept in self._concepts:
+                    new_concept = dict(concept)
+                    for attr_key in concept.keys():
+                        if attr_key not in allowed_attr_keys:
+                            new_concept.pop(attr_key)
+                    resource_list.append(new_concept)
+            else:
+                resource_list.append(self._concepts)
+
+        # Add mappings
+        if do_include_mappings:
+            if do_clean_for_bulk_import:
+                allowed_attr_keys = [
+                    'external_id', 'extras', 'from_concept_url', 'id', 'map_type', 'owner',
+                    'owner_type', 'retired', 'source', 'to_concept_code', 'to_concept_url',
+                    'to_source_url', 'to_concept_name', 'type']
+                for mapping in self._mappings:
+                    # Fix the mapping type, if its funky
+                    new_mapping = dict(mapping)
+                    if mapping['type'] == 'MappingVersion':
+                        new_mapping['type'] = 'Mapping'
+                    for attr_key in mapping.keys():
+                        if attr_key not in allowed_attr_keys:
+                            new_mapping.pop(attr_key)
+                    resource_list.append(new_mapping)
+            else:
+                resource_list.append(self._mappings)
+
+        # Add references
+        if do_include_references:
+            if 'references' in self._export_json and repo_type == 'Collection':
+                import re
+                compiled_regex = re.compile(r'^' + OclExport.RESOURCE_PATTERN + '$')
+                reference_expressions = []
+                for resource_reference in self._export_json['references']:
+                    # Make sure the expression does not have the resoruce version
+                    regex_result = compiled_regex.match(resource_reference['expression'])
+                    if regex_result is not None:
+                        reference_expressions.append(regex_result.group(1))
+                if reference_expressions:
+                    resource_list.append({
+                        'type': 'Reference',
+                        'owner': self._export_json['id'],
+                        'owner_type': self._export_json['type'],
+                        'data': {'expressions': reference_expressions}
+                    })
+
+        # Add resource for repository version
+        if do_include_repo_version:
+            allowed_attr_keys = ['type', 'id', 'description', 'released', 'retired', 'owner',
+                                 'owner_type', 'extras', 'external_id']
+            repo_version = {}
+            for attr_key in allowed_attr_keys:
+                repo_version[attr_key] = self._export_json.get(attr_key, '')
+            resource_list.append(repo_version)
+
         return resource_list
 
     def set_export(self, export_json=None, ocl_export=None):
@@ -177,7 +289,8 @@ class OclExport(object):
         self._concepts = self._export_json['concepts']
         self._mappings = self._export_json['mappings']
 
-    def _add_mappings_to_concept(self, concept, include_mappings=True, include_inverse_mappings=True):
+    def _add_mappings_to_concept(self, concept, include_mappings=True,
+                                 include_inverse_mappings=True):
         """ Adds mappings from this export to a copy of the provided concept """
         return_concept = concept.copy()
         return_concept['mappings'] = []
@@ -187,16 +300,18 @@ class OclExport(object):
                 return_concept['mappings'].append(mapping)
             if include_inverse_mappings and mapping['to_concept_url'] == concept_uri:
                 return_concept['mappings'].append(mapping)
-        return return_concept        
+        return return_concept
 
     def get_concept_by_index(self, index, include_mappings=False, include_inverse_mappings=False):
+        """ Return concept corresponding to a specified index """
         if not include_mappings and not include_inverse_mappings:
             return self._concepts[index]
         return self._add_mappings_to_concept(
             self._concepts[index], include_mappings=include_mappings,
             include_inverse_mappings=include_inverse_mappings)
 
-    def get_concept_by_id(self, concept_id, include_mappings=False, include_inverse_mappings=False):
+    def get_concept_by_id(self, concept_id, include_mappings=False,
+                          include_inverse_mappings=False):
         """ Returns the first concept that matches the specified ID, otherwise returns None """
         for concept in self._concepts:
             if concept['id'] == concept_id:
@@ -207,7 +322,8 @@ class OclExport(object):
                     include_inverse_mappings=include_inverse_mappings)
         return None
 
-    def get_concept_by_uri(self, concept_uri, include_mappings=False, include_inverse_mappings=False):
+    def get_concept_by_uri(self, concept_uri, include_mappings=False,
+                           include_inverse_mappings=False):
         """ Returns the first concept that matches the specified URL, otherwise returns None """
         for concept in self._concepts:
             if concept['url'] == concept_uri:
@@ -322,7 +438,8 @@ class OclExport(object):
             # Check for internal/external status of this mapping
             is_from_concept_in_export = False
             is_to_concept_in_export = False
-            if 'from_concept_url' in mapping and self.get_concept_by_uri(mapping['from_concept_url']):
+            if ('from_concept_url' in mapping and
+                    self.get_concept_by_uri(mapping['from_concept_url'])):
                 is_from_concept_in_export = True
             if 'to_concept_url' in mapping and self.get_concept_by_uri(mapping['to_concept_url']):
                 is_to_concept_in_export = True
