@@ -24,10 +24,12 @@ import json
 import sys
 import time
 from datetime import datetime
-import urllib
+
+import six
+import six.moves.urllib.parse as urllib_parse
 import requests
-import oclconstants
-import oclresourcelist
+from ocldev import oclconstants
+from ocldev import oclresourcelist
 
 
 class OclImportError(Exception):
@@ -198,7 +200,7 @@ class OclImportResults(object):
 
     def get_logging_keys(self):
         """ Returns list of logging keys stored in the results object """
-        return self._results.keys()
+        return list(self._results.keys())
 
     def get_import_results(self, results_mode='summary', root_key=None):
         """
@@ -239,7 +241,7 @@ class OclImportResults(object):
         if root_key:
             keys = [root_key]
         else:
-            keys = self._results.keys()
+            keys = list(self._results.keys())
 
         # Build results summary dictionary - organized by action type instead of root
         results_summary = {}
@@ -281,8 +283,9 @@ class OclImportResults(object):
         if root_key:
             output = '%s %s for key "%s"' % (process_str, output, root_key)
         else:
-            output = '%s %s of %s -- %s' % (
-                process_str, total_count, self.total_lines, output)
+            output = '%s %s of %s -- %s in %s secs' % (
+                process_str, total_count, self.total_lines, output, self.elapsed_seconds
+            )
 
         return output
 
@@ -294,7 +297,7 @@ class OclImportResults(object):
             logging_keys = [root_key]
             output = 'REPORT OF IMPORT RESULTS FOR KEY "%s":' % root_key
         else:
-            logging_keys = self._results.keys()
+            logging_keys = list(self._results.keys())
             output = 'REPORT OF IMPORT RESULTS:\n'
 
         # Iterate through logging keys and prepare report
@@ -329,7 +332,7 @@ class OclImportResults(object):
     @staticmethod
     def load_from_json(json_results):
         """ Load serialized JSON results into this object. Works with the to_json method """
-        if isinstance(json_results, basestring):
+        if isinstance(json_results, six.string_types):
             json_results = json.loads(json_results)
         if isinstance(json_results, dict):
             results_obj = OclImportResults()
@@ -356,6 +359,8 @@ class OclBulkImporter(object):
     oclfleximporter.OclBulkImporter()
     """
 
+    # when we get rid of py2 support use following
+    # OCL_BULK_IMPORT_API_ENDPOINT = '/importers/bulk-import/'
     OCL_BULK_IMPORT_API_ENDPOINT = '/manage/bulkimport/'
     OCL_BULK_IMPORT_MAX_WAIT_SECONDS = 120 * 60
     OCL_BULK_IMPORT_MINIMUM_DELAY_SECONDS = 5
@@ -403,6 +408,44 @@ class OclBulkImporter(object):
         # return task_id
         return import_response
 
+    @staticmethod
+    def get_queued_imports(api_url_root='', api_token='', queue='', status_filter=None):
+        """
+        :param api_url_root:
+        :param api_token:
+        :param queue: Optional queue filter. If empty, returns results across all user's queues.
+        :param status_filter: Optional status filter. Can be a string or list of strings
+            corresponding to values status values that are still returned.
+        """
+        # Retrieve the queued imports
+        url = '%s/manage/bulkimport/' % api_url_root
+        # when we get rid of py2 support use following
+        # url = '%s/importers/bulk-import/' % api_url_root
+
+        if queue:
+            url += '%s/' % queue
+        api_headers = {}
+        if api_token:
+            api_headers['Authorization'] = 'Token %s' % api_token
+        queued_imports_response = requests.get(url, headers=api_headers)
+        queued_imports_response.raise_for_status()
+        queued_imports = queued_imports_response.json()
+
+        # Optionally filter the queued imports by status
+        if status_filter:
+            filtered_queued_imports = []
+            if isinstance(status_filter, str):
+                status_filter = [status_filter]
+            if not isinstance(status_filter, list):
+                raise TypeError(
+                    "status_filter argument must be list or string. %s provided." % type(status_filter))
+            for queued_import in queued_imports:
+                if "state" in queued_import and queued_import["state"] in status_filter:
+                    filtered_queued_imports.append(queued_import)
+            return filtered_queued_imports
+
+        return queued_imports
+
     @staticmethod
     def get_bulk_import_results(task_id=None, api_url_root='', api_token='',
                                 max_wait_seconds=0, delay_seconds=15):
@@ -724,6 +767,7 @@ class OclFlexImporter(object):
                 self.input_list.append(json.loads(json_line_raw))
 
     def report_progress(self):
+        """ Save current progress string to instance """
         if self.update_progress:
             self.update_progress("%s of %s" % (
                 self.import_results.count,
@@ -750,7 +794,7 @@ class OclFlexImporter(object):
         # Loop through each JSON object
         self.import_results = OclImportResults(total_lines=total_lines)
         self.report_progress()
-        obj_def_keys = self.obj_def.keys()
+        obj_def_keys = list(self.obj_def.keys())
         count = 0
         num_processed = 0
         num_skipped = 0
@@ -789,6 +833,7 @@ class OclFlexImporter(object):
                 self.log('**** SKIPPING: %s' % message)
                 num_skipped += 1
 
+            self.import_results.elapsed_seconds = self.get_elapsed_seconds_from(start_time)
             self.log('[%s]' % self.import_results.get_detailed_summary())
             self.report_progress()
 
@@ -796,9 +841,13 @@ class OclFlexImporter(object):
             if self.import_delay and not self.test_mode:
                 time.sleep(self.import_delay)
 
-        self.import_results.elapsed_seconds = time.time() - start_time
+        self.import_results.elapsed_seconds = self.get_elapsed_seconds_from(start_time)
         return count
 
+    @staticmethod
+    def get_elapsed_seconds_from(start_time):
+        return time.time() - start_time
+
     def does_object_exist(self, obj_url, use_cache=True):
         """ Returns whether a resource at the specified URL already exists. """
 
@@ -811,7 +860,7 @@ class OclFlexImporter(object):
         if request_existence.status_code == requests.codes.ok:
             self._cache_obj_exists[obj_url] = True
             return True
-        elif request_existence.status_code >= 400 and request_existence.status_code < 500:
+        elif 400 <= request_existence.status_code < 500:
             return False
         else:
             raise UnexpectedStatusCodeError(
@@ -919,7 +968,7 @@ class OclFlexImporter(object):
 
     def process_reference_object(self, obj, batch_size=DEFAULT_REFERENCE_BATCH_SIZE):
         """ Process list of references, optionally splitting into batches """
-        batched_resources = OclFlexImporter.batch_reference_object(obj)
+        batched_resources = OclFlexImporter.batch_reference_object(obj, batch_size=batch_size)
         if len(batched_resources) > 1:
             message = ('INFO: New reference request with %s expressions automatically '
                        'split into %s batches of %s expressions or less') % (
@@ -928,7 +977,8 @@ class OclFlexImporter(object):
                            str(len(batched_resources[0]['data']['expressions'])))
             self.log(message)
         for batched_resource in batched_resources:
-            self.process_object(obj_type, new_obj)
+            self.process_object(
+                oclconstants.OclConstants.RESOURCE_TYPE_REFERENCE, batched_resource)
 
     def process_object(self, obj_type, obj):
         """ Processes an individual object in the import file """
@@ -1049,7 +1099,7 @@ class OclFlexImporter(object):
 
         # Pull out the fields that aren't allowed
         obj_not_allowed = {}
-        for k in obj.keys():
+        for k in list(obj.keys()):
             if k not in self.obj_def[obj_type]["allowed_fields"]:
                 obj_not_allowed[k] = obj.pop(k)
 
@@ -1208,7 +1258,7 @@ class OclFlexImporter(object):
 
         # Add query parameters (if provided)
         if query_params:
-            url += '?' + urllib.urlencode(query_params)
+            url += '?' + urllib_parse.urlencode(query_params)
 
         # If delete, then clear the cache of object existence
         if action_type == OclFlexImporter.ACTION_TYPE_DELETE:
@@ -1216,16 +1266,18 @@ class OclFlexImporter(object):
 
         # Import the object
         if not self.test_mode:
-            self.log(method, " ", self.api_url_root + url + '  ', json.dumps(obj))
+            endpoint_url = self.api_url_root + url
+            self.log(method, " ", endpoint_url + '  ', json.dumps(obj))
             if method == OclFlexImporter.HTTP_METHOD_POST:
                 request_result = requests.post(
-                    self.api_url_root + url, headers=self.api_headers, data=json.dumps(obj))
+                    endpoint_url, headers=self.api_headers, data=json.dumps(obj))
             elif method == OclFlexImporter.HTTP_METHOD_PUT:
                 request_result = requests.put(
-                    self.api_url_root + url, headers=self.api_headers, data=json.dumps(obj))
+                    endpoint_url, headers=self.api_headers, data=json.dumps(obj))
             elif method == OclFlexImporter.HTTP_METHOD_DELETE:
                 # NOTE: Deletes do not include the object in the request body
-                request_result = requests.delete(self.api_url_root + url, headers=self.api_headers)
+                endpoint_url += '&inline=true' if '?' in endpoint_url else '?inline=true'  # org non-async delete
+                request_result = requests.delete(endpoint_url, headers=self.api_headers)
             self.log("STATUS CODE:", request_result.status_code)
             self.log(request_result.headers)
             self.log(request_result.text)
@@ -1250,7 +1302,7 @@ class OclFlexImporter(object):
             if obj_url in self._cache_obj_exists:
                 del self._cache_obj_exists[obj_url]
         else:
-            for key in self._cache_obj_exists.keys():
+            for key in list(self._cache_obj_exists.keys()):
                 if key[:len(obj_url)] == obj_url:
                     del self._cache_obj_exists[key]
 
