@@ -2,9 +2,11 @@
 import csv
 import sys
 import json
-import oclconstants
-import oclvalidator
-import oclcsvtojsonconverter
+
+import six
+
+from ocldev import oclconstants
+from ocldev import oclcsvtojsonconverter
 
 
 class OclResourceList(object):
@@ -42,7 +44,7 @@ class OclResourceList(object):
         """ Return whether the two objects are equal, i.e. have the same resource lists """
         if len(self) != len(other):
             return False
-        for i in range(len(self)):
+        for i in six.moves.range(len(self)):
             if self[i] != other[i]:
                 return False
         return True
@@ -56,13 +58,14 @@ class OclResourceList(object):
         Return the OclResourceList as a list of OclResourceList objects with maximum
         size of chunk_size.
         """
-        chunked_lists = [self._resources[i:i + chunk_size] for i in xrange(0, len(self._resources), chunk_size)]
+        chunked_lists = [
+            self._resources[i:i + chunk_size] for i in six.moves.range(0, len(self._resources), chunk_size)]
         chunked_resource_lists = []
         for chunked_list in chunked_lists:
             chunked_resource_lists.append(OclResourceList(chunked_list))
         return chunked_resource_lists
 
-    def next(self):
+    def __next__(self):
         """ Get the next item in the list """
         if self._current_iter >= len(self._resources):
             raise StopIteration
@@ -70,6 +73,9 @@ class OclResourceList(object):
             self._current_iter += 1
             return self._resources[self._current_iter - 1]
 
+    def next(self):
+        return self.__next__()
+
     def refresh_index(self):
         """
         Refresh the URL index. This is required if resource ID and URL-related fields are edited
@@ -270,7 +276,7 @@ class OclResourceList(object):
 
     def get_resource_by_url(self, url):
         """ Return the first resource that matches the specified URL. """
-        if isinstance(url, basestring) and url:
+        if isinstance(url, six.string_types) and url:
             url_needle = url.strip()
         else:
             return None
@@ -326,6 +332,7 @@ class OclCsvResourceList(OclResourceList):
 
     def validate(self):
         """ Validate the resource list using the OclCsvValidator """
+        from ocldev import oclvalidator
         oclvalidator.OclCsvValidator.validate(self)
 
     def get_unique_column_headers(self, default_columns=None):
@@ -358,6 +365,7 @@ class OclJsonResourceList(OclResourceList):
 
     def validate(self):
         """ Validate the resource list using the OclCsvValidator """
+        from ocldev import oclvalidator
         oclvalidator.OclJsonValidator.validate(self)
 
     def get_unique_column_headers(self, default_columns=None):
